@@ -44,7 +44,7 @@ StatsQSOsPerDXCCBarChartWidget::StatsQSOsPerDXCCBarChartWidget(DataProxy_SQLite
 
 StatsQSOsPerDXCCBarChartWidget::~StatsQSOsPerDXCCBarChartWidget()
 {
-    delete(dataProxy);
+   //delete(dataProxy);
 }
 
 void StatsQSOsPerDXCCBarChartWidget::createUI()
@@ -58,7 +58,6 @@ void StatsQSOsPerDXCCBarChartWidget::createUI()
 
 void StatsQSOsPerDXCCBarChartWidget::prepareChart(const int _log)
 {
-
     QChart *chart = new QChart();
     chart->setAnimationOptions(QChart::SeriesAnimations);
     chart->legend()->setVisible(true);
