@@ -29,7 +29,7 @@
 
 
 
-FileManager::FileManager(DataProxy_SQLite *dp)
+FileManager::FileManager(DataProxy_SQLite *dp, World *injectedWorld)
 //FileManager::FileManager(const QString &_klogDir, const QString &_softVersion, DataBase _db)
 {
      //qDebug() << Q_FUNC_INFO << " -3: Dir(2)" << _klogDir;
@@ -37,8 +37,9 @@ FileManager::FileManager(DataProxy_SQLite *dp)
     util = new Utilities(Q_FUNC_INFO);
     //util->setLongPrefixes(dataProxy->getLongPrefixes());
     db = new DataBase(Q_FUNC_INFO, klogVersion, util->getKLogDBFile());
-    world = new World(dataProxy, Q_FUNC_INFO);
-    awards = new Awards(dataProxy, Q_FUNC_INFO);
+    //world = new World(dataProxy, Q_FUNC_INFO);
+    //world = injectedWorld;
+    awards = new Awards(dataProxy, injectedWorld, Q_FUNC_INFO);
     //qDebug() << Q_FUNC_INFO << " -3: Dir(2) - END" ;
 }
 
@@ -46,7 +47,7 @@ FileManager::~FileManager()
 {
     delete(db);
     delete(awards);
-    delete(world);
+    //delete(world);
 }
 
 void FileManager::init()
@@ -164,272 +165,11 @@ void FileManager::setSendQSLByDefault (const bool _send)
     sendEQSLByDefault = _send;
 }
 
-QList<int> FileManager::adifLogExportReturnList(const QString& _fileName, const QString &_callsign, const QString &_grid, const QDate &_startDate, const QDate &_endDate, const int _logN, const ExportMode _em)
-{
-    //qDebug() << Q_FUNC_INFO << ": Start)" << _fileName << "/" << _callsign << "/ " << _grid;
-    QList<int> qsos;
-    qsos.clear();
-    Callsign call(_callsign);
-    if ((!call.isValid()) && (_callsign != "ALL") && (_callsign !="NOT"))
-    {
-         showError(tr("The selected callsign (%1) is not valid, please check it again to export the log.").arg(_callsign));
-         return qsos;
-    }
-    noMoreQso = false;
-    if (_logN != -1)
-    { // We will export data from ALL logs.
-        if (!dataProxy->doesThisLogExist(_logN))
-        {
-            //qDebug() << Q_FUNC_INFO << ": The log does not exist";
-            showError(tr("The selected log does not exist, please check it again."));
-            return qsos;
-        }
-    }
-
-    QString queryStringCount;
-    QString queryString;
-    QString _queryStation;
-    QString _queryGrid;
-
-    if (_callsign == "NOT")
-    {
-         //qDebug() << Q_FUNC_INFO << ": else: " << _callsign;
-         _queryStation = QString(" ((station_callsign ='') OR (station_callsign IS NULL))");
-    }
-    else if (_callsign == "ALL")
-    {
-         //qDebug() << Q_FUNC_INFO << ": ALL: " << _callsign;
-        _queryStation = QString(" station_callsign !='ALL'");
-    }
-    else
-    {
-         //qDebug() << Q_FUNC_INFO << ": ValidCall: " << _callsign;
-        _queryStation = QString(" station_callsign ='%1'").arg(_callsign);
-    }
-    //qDebug() << Q_FUNC_INFO << ": _queryStation: " << _queryStation;
-    if (util->isValidGrid(_grid))
-    {
-        _queryGrid = QString(" AND my_gridsquare = '%1'").arg(_grid);
-    }
-    else if (_grid == "NOT")
-    {
-        _queryGrid = QString(" AND (my_gridsquare = '' OR my_gridsquare IS NULL)");
-    }
-    else
-    {
-        _queryGrid = QString();
-    }
-
-    QString _queryDateFrom;
-
-    if (_startDate.isValid())
-    {
-         _queryDateFrom = QString(" AND qso_date >= '%1'").arg(util->getDateSQLiteStringFromDate(_startDate));
-    }
-    else
-    {
-         _queryDateFrom = QString(" AND qso_date != '1'");
-    }
-
-    QString _queryDateTo;
-    if (_endDate.isValid())
-    {
-         _queryDateTo = QString(" AND qso_date <= '%1'").arg(util->getDateSQLiteStringFromDate(_endDate.addDays(1)));
-    }
-    else
-    {
-         _queryDateTo = QString();
-    }
-
-    QString _queryLog;
-    if (_logN == -1)
-    {
-         _queryLog = QString(" AND logNumber = '%1'").arg(_logN);
-    }
-    else
-    {
-         _queryLog = QString();
-    }
-
-    QFile file(_fileName);
-    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) /* Flawfinder: ignore */
-    {
-         showError(tr("The file %1 can't be opened.").arg(_fileName));
-         return qsos;
-    }
-
-    QSqlQuery query;
-
-    if (_em == ModeLotW)
-    {
-         //qDebug() << Q_FUNC_INFO << ":  Exporting for LoTW";
-        // LoTW Required fields: call sign, UTC Date, UTC time, Mode, Band
-        // LoTW Optional fields: RX band, Frecuency TX, frecuency RX, Propagation mode, Satellite
-
-        queryStringCount = QString("SELECT COUNT (id) FROM log WHERE") + _queryStation + _queryGrid + QString(" AND lotw_qsl_sent='Q'") + _queryDateFrom + _queryDateTo;
-        queryString = QString("SELECT id, call, freq, bandid, band_rx, freq_rx, modeid, gridsquare, my_gridsquare, qso_date, prop_mode, sat_name, station_callsign FROM log WHERE") + _queryStation + QString(" AND lotw_qsl_sent='Q'") + _queryDateFrom + _queryDateTo;
-    }
-    else if (_em == ModeClubLog)
-    {
-         //qDebug() << Q_FUNC_INFO << ": Exporting for ClubLog";
-        queryStringCount = QString("SELECT COUNT (id) FROM log WHERE") + _queryStation + QString(" AND clublog_qso_upload_status='M'") + _queryDateFrom + _queryDateTo;
-        queryString = QString("SELECT id, call, rst_sent, rst_rcvd, freq, bandid, band_rx, modeid, qso_date, qsl_rcvd, qslrdate, qslsdate, prop_mode, operator, station_callsign, dxcc, qsl_sent, lotw_qsl_rcvd, credit_granted, notes, qso_date_off FROM log WHERE") + _queryStation + QString(" AND clublog_qso_upload_status='M'") + _queryDateFrom + _queryDateTo;
-    }
-    else if (_em == ModeEQSL)
-    {
-         //qDebug() << Q_FUNC_INFO << ":  Exporting for eQSL";
-        queryStringCount = QString("SELECT COUNT (id) FROM log WHERE") + _queryStation + QString(" AND eqsl_qsl_sent='Q'") + _queryDateFrom + _queryDateTo;
-        queryString = QString("SELECT id, call, rst_sent, freq, bandid, modeid, qso_date, prop_mode, operator, station_callsign, sat_name, my_cnty, my_gridsquare, my_lat, my_lon FROM log WHERE") + _queryStation + QString(" AND eqsl_qsl_sent='Q'") + _queryDateFrom + _queryDateTo;
-    }
-    else
-    {
-        //qDebug() << Q_FUNC_INFO << ": Exporting normal ADIF";
-        if (_callsign == "ALL")
-        {
-            queryStringCount = QString("SELECT COUNT (id) FROM log");
-            queryString = QString("SELECT * FROM log");
-        }
-        else if (_callsign == "NOT")
-        {
-            queryStringCount = QString("SELECT COUNT (id) FROM log WHERE ") + _queryStation + _queryDateFrom + _queryDateTo + _queryLog;
-            queryString = QString("SELECT * FROM log WHERE ") + _queryStation + _queryDateFrom + _queryDateTo + _queryLog;
-        }
-        else
-        {
-            queryStringCount = QString("SELECT COUNT (id) FROM log WHERE") + _queryStation  + _queryDateFrom + _queryDateTo + _queryLog;
-            queryString = QString("SELECT * FROM log WHERE") + _queryStation + _queryDateFrom + _queryDateTo + _queryLog;
-        }
-    }
-    //qDebug() << Q_FUNC_INFO << ":  Query Count: " << queryStringCount;
-    //qDebug() << Q_FUNC_INFO << ":  Query data: " << queryString;
-    int numberOfQsos = dataProxy->getHowManyQSOInLog(_logN);
-    int i = 0;
-
-    bool sqlOK = query.exec(queryStringCount);
-    //qDebug() << Q_FUNC_INFO << ": " << query.lastQuery();
-    if (!sqlOK)
-    {
-        //qDebug() << Q_FUNC_INFO << ":  Query Error" ;
-        emit queryError(Q_FUNC_INFO, query.lastError().databaseText(), query.lastError().text(), query.lastQuery());
-        return qsos;
-    }
-    else
-    {
-        query.next();
-        if (query.isValid())
-        {
-            i = (query.value(0)).toInt();
-            if (i>0)
-            {
-                numberOfQsos = i;
-                query.finish();
-            }
-            else if (i == 0)
-            {
-                showError(tr("There are no QSOs pending to be exported with that station callsign."));
-                return qsos;
-            }
-        }
-    }
-
-    QTextStream out(&file);
-
-    int step = util->getProgresStepForDialog(numberOfQsos);
-    //qDebug() << Q_FUNC_INFO << ":  " << QString::number(step);
-
-    QProgressDialog progress(tr("Writing ADIF file..."), tr("Abort writing"), 0, numberOfQsos, this);
-    progress.setMaximum(numberOfQsos);
-    progress.setWindowModality(Qt::WindowModal);
-    progress.setValue(0);
-    progress.setWindowTitle(tr("Export"));
-    progress.setAutoClose(true);
-
-
-    writeADIFHeader(out, _em, numberOfQsos);
-
-    i = 0;
-
-    sqlOK = query.exec(queryString);
-     //qDebug() << Q_FUNC_INFO << ":  " << query.lastQuery();
-    if (!sqlOK)
-    {
-         //qDebug() << Q_FUNC_INFO << ":  Query Error" ;
-        emit queryError(Q_FUNC_INFO, query.lastError().databaseText(), query.lastError().text(), query.lastQuery());
-        return qsos;
-    }
-
-
-    int nameCol;
-
-    //qDebug() << Q_FUNC_INFO << ":  Entering the While..." ;
-    QSqlRecord rec;
-    while ( (query.next()) && (!noMoreQso) )
-    {
-        //qDebug() << Q_FUNC_INFO << ":  Start of While" ;
-        if (query.isValid())
-        {
-            rec = query.record();
-            nameCol = rec.indexOf("id");
-            qsos.append((query.value(nameCol)).toInt());
-            writeQuery(rec, out, _em, false, false, _logN); // JustMarked = false, onlyRequested = false
-            //Use std::move(query) instead of query?
-            // Copy queries is deprecated
-
-            i++;
-            //qDebug() << Q_FUNC_INFO << ":  Start of isValid" ;
-        } // END of if (query.isValid())
-        else
-        {
-            //qDebug() << Q_FUNC_INFO << ":  Querty NOT isValid" ;
-        }
-
-        //qDebug() << Q_FUNC_INFO << ":  Checking if cancelled" ;
-
-        if (( (i % step ) == 0) )
-        { // To update the speed I will only show the progress once each X QSOs
-            //qDebug() << Q_FUNC_INFO << ":  ********************************   UPDATING THE MESSAGE! " << QString::number(i) ;
-            QString aux = tr("Exporting ADIF file...\n QSO: %1 / %2 ").arg(i).arg(numberOfQsos);
-            progress.setLabelText(aux);
-            progress.setValue(i);
-            //qDebug() << Q_FUNC_INFO << ":  ********************************   UPDATING THE MESSAGE: " << aux ;
-        }
-
-        if ( progress.wasCanceled() )
-        {
-            QMessageBox msgBox;
-            msgBox.setWindowTitle(tr("KLog - User cancelled"));
-            QString aux = QString(tr("You have canceled the file export. The file will be removed and no data will be exported.") + "\n" + tr("Do you still want to cancel?"));
-            msgBox.setText(aux);
-            msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
-            msgBox.setDefaultButton(QMessageBox::No);
-            int ret = msgBox.exec();
-            switch (ret)
-            {
-                case QMessageBox::Yes:
-            // Yes was clicked
-                noMoreQso = true;
-                qsos.clear();
-                progress.setValue(numberOfQsos);
-                return qsos;
-                case QMessageBox::No:
-                // No Save was clicked
-                break;
-                default:
-                // should never be reached
-                break;
-            }
-        }
-    } // END OF WHILE
-    //qDebug() << Q_FUNC_INFO << ":  End: " << QString::number(qsos.count());
-    progress.setValue(numberOfQsos);
-    return qsos;
-}
-
-QList<int> FileManager::adifLogExportReturnList2(const QString& _fileName, const QString &_callsign, QList<int> _qsos, const ExportMode _em, const int _logN)
+QList<int> FileManager::adifLogExportReturnList(const QString& _fileName, const QString &_callsign, QList<int> _qsos, const ExportMode _em, const int _logN)
 {
     Q_UNUSED(_logN);
-    //qDebug() << Q_FUNC_INFO << " - Start";
-    //qDebug() << Q_FUNC_INFO << " - QSOs: " << QString::number(_qsos.length ());
+   //qDebug() << Q_FUNC_INFO << " - Start";
+   //qDebug() << Q_FUNC_INFO << " - QSOs: " << QString::number(_qsos.length ());
     //qDebug() << Q_FUNC_INFO << ": Start)" << _fileName << "/" << _callsign << "/ " << _grid;
     QList<int> qsos;
     qsos.clear();
@@ -444,15 +184,15 @@ QList<int> FileManager::adifLogExportReturnList2(const QString& _fileName, const
     QString queryString;
     if (_em == ModeLotW)
     {
-        queryString = QString("call, freq, bandid, band_rx, freq_rx, modeid, gridsquare, my_gridsquare, qso_date, prop_mode, sat_name, station_callsign");
+        queryString = QString("id, call, freq, bandid, band_rx, freq_rx, modeid, gridsquare, my_gridsquare, qso_date, prop_mode, sat_name, station_callsign");
     }
     else if (_em == ModeClubLog)
     {
-         queryString = QString("call, rst_sent, rst_rcvd, freq, bandid, band_rx, modeid, qso_date, qsl_rcvd, qslrdate, qslsdate, prop_mode, operator, station_callsign, dxcc, qsl_sent, lotw_qsl_rcvd, credit_granted, notes, qso_date_off");
+         queryString = QString("id, call, rst_sent, rst_rcvd, freq, bandid, band_rx, modeid, qso_date, qsl_rcvd, qslrdate, qslsdate, prop_mode, operator, station_callsign, dxcc, qsl_sent, lotw_qsl_rcvd, credit_granted, notes, qso_date_off");
     }
     else if (_em == ModeEQSL)
     {
-        queryString = QString("call, rst_sent, freq, bandid, modeid, submode, qso_date, prop_mode, operator, station_callsign, sat_name, my_cnty, my_gridsquare, my_lat, my_lon, qslmsg, sat_mode, sat_name");
+        queryString = QString("id, call, rst_sent, freq, bandid, modeid, submode, qso_date, prop_mode, operator, station_callsign, my_cnty, my_gridsquare, my_lat, my_lon, qslmsg, sat_mode, sat_name");
     }
     else
     { // THis is ModeADIF and ModeQRZ we will upload ALL the ADIF files
@@ -471,105 +211,6 @@ QList<int> FileManager::adifLogExportReturnList2(const QString& _fileName, const
     return qsos;
 }
 
-bool FileManager::adifQSOsExport(const QString& _fileName, QList<int> _qsos)
-{
-   //qDebug() << Q_FUNC_INFO << _fileName;
-    int numberOfQSOs = _qsos.length();
-    if (numberOfQSOs<1)
-    {
-        //TODO: Warn the user NO QSOS TO EXPORT
-       //qDebug() << "FileManager::adifQSOsExport: No QSOs received to be exported";
-    }
-    noMoreQso = false;
-   //qDebug() << Q_FUNC_INFO << "  - 01";
-    QFile file(_fileName);
-    if (!file.open(QIODevice::WriteOnly | QIODevice::Text)) /* Flawfinder: ignore */
-        return false;
-   //qDebug() << Q_FUNC_INFO << "  - 02";
-    QTextStream out(&file);
-
-    QSqlQuery query;
-    //qDebug() << Q_FUNC_INFO << "  - 10";
-    QString numbers = QString();
-    foreach (int i, _qsos)
-    {
-        //qDebug() << Q_FUNC_INFO << "  - query: " << QString("id = '%1'").arg(i) ;
-        numbers = numbers + QString::number(i) ;
-        if (i != _qsos.last())
-        {
-           numbers = numbers + ", ";
-        }
-    }
-
-    QString queryString = QString("SELECT * FROM log WHERE id IN (%1)").arg(numbers);
-    //qDebug() << Q_FUNC_INFO << " - writing the header";
-    writeADIFHeader(out, ModeADIF, _qsos.length());
-   //qDebug() << Q_FUNC_INFO << " - writing the body";
-
-    bool sqlOK = query.exec(queryString);
-    if (!sqlOK)
-    {
-    //qDebug() << Q_FUNC_INFO << " - query error: " << query.lastQuery();
-     emit queryError(Q_FUNC_INFO, query.lastError().databaseText(), query.lastError().text(), query.lastQuery());
-    }
-   //qDebug() << "FileManager::adifQSOsExport: query: " << query.lastQuery();
-    QProgressDialog progress(tr("Writing ADIF file..."), tr("Abort writing"), 0, numberOfQSOs, this);
-    progress.setMaximum(numberOfQSOs);
-    progress.setWindowModality(Qt::ApplicationModal);
-    progress.setWindowTitle(tr("Export progress"));
-    int currentQso = 0;
-    int step = util->getProgresStepForDialog(numberOfQSOs);
-    QSqlRecord rec;
-    while ( (query.next()) && (!noMoreQso) )
-    {
-       //qDebug() << "FileManager::adifLogExportToFile -  Just in the While";
-        if (query.isValid())
-        {
-            rec = query.record();
-            //qDebug() << "FileManager::adifLogExportToFile -  Query is Valid";
-             writeQuery(rec, out, ModeADIF, false, false, -1);
-        } // Closes the isValid
-        else {
-           //qDebug() << "FileManager::adifLogExportToFile -  Query is NOT Valid";
-        }
-
-       //qDebug() << "FileManager::adifLogExportToFile -  before showing progress: " << QString::number(currentQso);
-        currentQso++;
-
-        if (( (currentQso % step ) == 0) )
-        { // To update the speed I will only show the progress once each X QSOs
-           //qDebug() << "FileManager::adifLogExportToFile -  Showing progress: " ;
-            QString aux1 = tr("Writing ADIF file...\n QSO: ")  + QString::number(currentQso) + "/" + QString::number(numberOfQSOs);
-            progress.setLabelText(aux1);
-            progress.setValue(currentQso);
-        }
-       //qDebug() << "FileManager::adifLogExportToFile -  after showing progress (current%Step): " << QString::number(currentQso%step);
-        if ( progress.wasCanceled() )
-        {
-            QMessageBox msgBox;
-            msgBox.setWindowTitle(tr("KLog - User cancelled"));
-            QString aux = QString(tr("You have canceled the file export. The file will be removed and no data will be exported.") + "\n" + tr("Do you still want to cancel?"));
-            msgBox.setText(aux);
-            msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
-            msgBox.setDefaultButton(QMessageBox::No);
-            int ret = msgBox.exec();
-            switch (ret) {
-              case QMessageBox::Yes:
-                  // Yes was clicked
-                    noMoreQso = true;
-                  break;
-              case QMessageBox::No:
-                    // No Save was clicked
-                  break;
-              default:
-                    // should never be reached
-                  break;
-            }
-        }
-    }
-   //qDebug() << "FileManager::adifQSOsExport - END";
-    return true;
-}
 
 bool FileManager::adifQSOsExport2(const QString& _fileName, const QString& _fields, QList<int> _qsos, ExportMode _em)
 { // The fields are the database fields that are to be selected in the query
@@ -622,7 +263,7 @@ bool FileManager::adifQSOsExport2(const QString& _fileName, const QString& _fiel
     }
     else
     {
-        queryString = QString("SELECT %1 FROM log WHERE id IN (%2)").arg(fields).arg(numbers);
+        queryString = QString("SELECT %1 FROM log WHERE id IN (%2)").arg(fields, numbers);
     }
     //qDebug() << Q_FUNC_INFO << " - writing the header";
     writeADIFHeader(out, _em, numberOfQSOs);
@@ -693,7 +334,6 @@ bool FileManager::adifQSOsExport2(const QString& _fileName, const QString& _fiel
     return true;
 }
 
-
 bool FileManager::adifLogExportToFile(const QString& _fileName, const int _logN, bool justMarked, bool _qslRequested , bool _lotw)
 {
     //adifLogExportToFile(const QString& _fileName, const int _logN=0, bool justMarked = false, bool _qslRequested = false, bool _lotw=false);
@@ -762,7 +402,7 @@ bool FileManager::adifLogExportToFile(const QString& _fileName, const int _logN,
                 numberOfQsos = query.value(0).toInt();
             }
         }
-           //qDebug() << "FileManager::adifLogExportToFile -  numberOfQsos = " << QString::number(numberOfQsos)<< QT_ENDL;
+           //qDebug() << Q_FUNC_INFO << " -    numberOfQsos = " << QString::number(numberOfQsos)<< QT_ENDL;
     }
     else
     {
@@ -786,10 +426,10 @@ bool FileManager::adifLogExportToFile(const QString& _fileName, const int _logN,
         {
             numberOfQsos = (query1.value(0)).toInt();
         }
-          //qDebug() << "FileManager::adifLogExportToFile -  numberOfQsos = " << QString::number(numberOfQsos)<< QT_ENDL;
+          //qDebug() << Q_FUNC_INFO << " -    numberOfQsos = " << QString::number(numberOfQsos)<< QT_ENDL;
     }
     //qDebug() << Q_FUNC_INFO << " - 30";
-    //qDebug() << "FileManager::adifLogExportToFile - end get numberOfQsos = " << QString::number(numberOfQsos);
+    //qDebug() << Q_FUNC_INFO << " -   end get numberOfQsos = " << QString::number(numberOfQsos);
 
     step = util->getProgresStepForDialog(numberOfQsos);
     //qDebug() << Q_FUNC_INFO << " - 31";
@@ -847,17 +487,17 @@ bool FileManager::adifLogExportToFile(const QString& _fileName, const int _logN,
             //qDebug() << Q_FUNC_INFO << " - 50.2";
         } // Closes the isValid
 
-        //qDebug() << "FileManager::adifLogExportToFile -  before showing progress: " << QString::number(currentQso);
+        //qDebug() << Q_FUNC_INFO << " -    before showing progress: " << QString::number(currentQso);
         currentQso++;
         if (( (currentQso % step )== 0) )
         { // To update the speed I will only show the progress once each X QSOs
-            //qDebug() << "FileManager::adifLogExportToFile -  Showing progress: " ;
+            //qDebug() << Q_FUNC_INFO << " -    Showing progress: " ;
             aux1 = tr("Writing ADIF file...\n QSO: ")  + QString::number(currentQso) + "/" + QString::number(numberOfQsos);
             progress.setLabelText(aux1);
             progress.setValue(currentQso);
             //qDebug() << Q_FUNC_INFO << " - 50.3";
         }
-        //qDebug() << "FileManager::adifLogExportToFile -  after showing progress (current%Step): " << QString::number(currentQso%step);
+        //qDebug() << Q_FUNC_INFO << " -    after showing progress (current%Step): " << QString::number(currentQso%step);
 
         if ( progress.wasCanceled() )
         {
@@ -884,20 +524,20 @@ bool FileManager::adifLogExportToFile(const QString& _fileName, const int _logN,
         }
     } //Closes the While
     //qDebug() << Q_FUNC_INFO << " - 60";
-      //qDebug() << "FileManager::adifLogExportToFile -  after the While";
+      //qDebug() << Q_FUNC_INFO << " -    after the While";
     progress.setValue(numberOfQsos);
     //qDebug() << Q_FUNC_INFO << " - 70";
     if (noMoreQso)
     {
     //TODO: Remove the file (_fileName)
-          //qDebug() << "FileManager::adifLogExportToFile -  noMoreQSO = true";
+          //qDebug() << Q_FUNC_INFO << " -    noMoreQSO = true";
         file.remove();
         //qDebug() << Q_FUNC_INFO << " - 70.1";
         return false;
     }
     else
     {
-          //qDebug() << "FileManager::adifLogExportToFile -  noMoreQSO = false";
+          //qDebug() << Q_FUNC_INFO << " -    noMoreQSO = false";
         //qDebug() << Q_FUNC_INFO << " - 70.2";
         return writeBackupDate();
     }
@@ -924,7 +564,7 @@ bool FileManager::adifLogExportMarked(const QString& _fileName)
 
 int FileManager::adifLoTWReadLog2(const QString& fileName, const int logN)
 {
-  //qDebug() << Q_FUNC_INFO << " - " << fileName;
+    //qDebug() << Q_FUNC_INFO << " - " << fileName;
    //QSO qso(Q_FUNC_INFO);
    QString stationCallSign;
    stationCallSign.clear();
@@ -983,132 +623,194 @@ int FileManager::adifLoTWReadLog2(const QString& fileName, const int logN)
     }
     file.seek(pos);
     //qDebug() << Q_FUNC_INFO << " - END" ;
-    return adifReadLog2(fileName, stationCallSign, logN);
+    return adifReadLog(fileName, stationCallSign, logN);
 }
 
-int FileManager::adifReadLog2(const QString& tfileName, QString _stationCallsign, int logN)
+bool FileManager::isALoTWDownloadedFile(QFile & _f)
 {
-    //qDebug() << Q_FUNC_INFO << " - Start: " << tfileName << "/" << QString::number(logN);
-    QFile file( tfileName );
-    if (!file.exists ())
-    {
-        //qDebug() << Q_FUNC_INFO << " - END: file does not exist";
-        return false;
-    }
-    int qsos = howManyQSOsInFile (file);
-    //qDebug() << Q_FUNC_INFO << " - QSOs: " << QString::number(qsos);
-    qint64 pos = passHeader (file); // Position in the file to calculate where the header ends
+    //qDebug() << Q_FUNC_INFO << " - Start";
+    //qDebug() << Q_FUNC_INFO << " - Start: " << _f.fileName ();
+
+    QString programId = getProgramID(_f);
+    if (programId == "LOTW")
+        return true;
+
+    return false;
+    //QString line = file.readLine().trimmed();
+    //bool isLoTWFile = (line == QString("ARRL Logbook of the World Status Report"));
+
+    //file.close();
+    //qDebug() << Q_FUNC_INFO << " - LOTW: " << util->boolToQString(isLoTWFile);
+
+    //return isLoTWFile;
+}
+
+int FileManager::adifReadLog(const QString& tfileName, QString _stationCallsign, int logN)
+{
+   //qDebug() << Q_FUNC_INFO << " - Start: " << tfileName << "/" << QString::number(logN);
+    QFile file(tfileName);
+    if (!file.exists())
+        return -1;
+
+    bool lotWDownloaded = isALoTWDownloadedFile(file);
+    //if (lotWDownloaded)
+        //qDebug() << Q_FUNC_INFO << " - LoTW file detected!";
+
+    int qsos = howManyQSOsInFile(file);
+    qint64 pos = passHeader(file);
+
     if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) /* Flawfinder: ignore */
-    {
-        //qDebug() << Q_FUNC_INFO << "  File not found" ;
-        return false;
-    }
+        return -2;
 
-    file.seek (pos); // QSO Data starts here
-
-    QSO qso;
-    QStringList fields; // fields keeps the running array,
-
-    fields.clear ();
-    QString line = QString();
-    //line.clear();
-    qso.clear ();
-
-    //qDebug() << Q_FUNC_INFO << ": Progress defined" ;
+    file.seek(pos);
     QProgressDialog progress(tr("Reading ADIF file..."), tr("Abort reading"), 0, qsos, this);
-    progress.setMaximum(qsos);
     progress.setWindowModality(Qt::ApplicationModal);
-    progress.setValue(0);
-    //progress.setWindowTitle(tr("Import"));
     progress.setAutoClose(true);
+
     int step = util->getProgresStepForDialog(qsos);
     int i = 0;
     bool noMoreQSO = false;
-    //qDebug() << Q_FUNC_INFO << ": We start the while" ;
-    while ((!file.atEnd()) && (!noMoreQSO))
+
+    QSO qso;
+    QStringList fields;
+    QTime startTime = QTime::currentTime();
+   //qDebug() << Q_FUNC_INFO << " - Starting to read the file";
+    while (!file.atEnd() && !noMoreQSO)
     {
-        // One line is read and splitted into the list of fields
-        // Fields are analyzed one by one and extracted from the list of fields
-        // Fields are added to the QSO
-        // If the field is <EOR> the QSO is completed and added to the log.
-        // Once the QSO is added to the log, QSO is cleared and process continues
-        // Once the list of fields is empty, we read another file and start again
-        // until we reach the end of file
-        line.clear();
-        line.append(file.readLine().trimmed().toUpper());
-        //qDebug() << Q_FUNC_INFO << ": Reading the line: " << line ;
-        fields << line.split("<", QT_SKIP);
+        QString line = file.readLine().trimmed().toUpper();
+        if (line.isEmpty())
+            continue;
+
+        fields << line.split("<", Qt::SkipEmptyParts);
+        QString progressText;
         while (!fields.isEmpty())
         {
-            //qDebug() << Q_FUNC_INFO << QString(": Fields still has %1 items").arg(fields.count()) ;
-            QString fieldToAnalyze = "<" + (fields.takeFirst()).trimmed();
-            //qDebug() << Q_FUNC_INFO << QString(": Extracted: %1").arg(fieldToAnalyze) ;
-            if ((fieldToAnalyze.contains ("<EOR>")) || (fieldToAnalyze.contains("<APP_LOTW_EOF>")) )
+            QString fieldToAnalyze = "<" + fields.takeFirst().trimmed();
+
+            if (fieldToAnalyze.contains("<EOR>") || fieldToAnalyze.contains("<APP_LOTW_EOF>"))
             {
-                //qDebug() << Q_FUNC_INFO << QString(": EOR detected, QSO added");
-                qso.setLogId (logN);
-                Callsign call1(_stationCallsign);
-                Callsign call2(qso.getStationCallsign());
-                if ((call1.isValid()) && (!call2.isValid()))
-                {
-                    qso.setStationCallsign(_stationCallsign);
-                }
-                qso.toDB ();
-                qso.clear ();
-                i++;
-                if (( (i % step ) == 0) )
-                { // To update the speed I will only show the progress once each X QSOs
-                    //qDebug() << Q_FUNC_INFO << " MOD 0 - i = " << QString::number(i) ;
-                    QString aux = tr("Importing ADIF file...") + "\n" + tr(" QSO: ")  + QString::number(i) + "/" + QString::number(qsos);
-                    progress.setLabelText(aux);
-                    progress.setValue(i);
+                qso.setLogId(logN);
+                qso.setLoTWUpdating(lotWDownloaded);
+                if (processQSO(qso, _stationCallsign) > 0){
+                    ++i;
                 }
                 else
                 {
-                    //qDebug() << Q_FUNC_INFO << " Mod: "<< QString::number(i) << " mod " << QString::number(step) << " = " << QString::number(i % step);
+                 //qDebug() << Q_FUNC_INFO << " - QSO not added! processQSO failed";
+                }
+                qso.clear();
+
+
+                if (i % step == 0)
+                {
+                    //qDebug() << Q_FUNC_INFO << " - QSOS: " << i ;
+                    //qDebug() << Q_FUNC_INFO << " - secs: " << startTime.secsTo(QTime::currentTime());
+                    //qDebug() << Q_FUNC_INFO << " - qsos/sec: " << i / startTime.secsTo(QTime::currentTime());
+                    //if (startTime.secsTo(QTime::currentTime()) >0)
+                    //    progressText = QString("Importing ADIF file ... \nQSO: %1 / %2 \nSpeed: %3 QSOs/sec").arg(i, qsos, i / startTime.secsTo(QTime::currentTime()));
+                    //progress.setLabelText(progressText);
+                    progress.setLabelText(tr("Importing ADIF file...") + "\n" + tr(" QSO: ") + QString::number(i) + "/" + QString::number(qsos));
+                    progress.setValue(i);
                 }
             }
             else
             {
-                //qDebug() << Q_FUNC_INFO << QString(": Adding this to the QSO: %1").arg(fieldToAnalyze) ;
-                //fieldToAnalyze must be an ADIF record: <Field:length:Data type>Data                
-                qso.setData (fieldToAnalyze);
+                qso.setData(fieldToAnalyze, lotWDownloaded);
             }
-            if ( progress.wasCanceled() )
+
+            if (progress.wasCanceled())
             {
-                //qDebug() << Q_FUNC_INFO << QString(": Progress Cancelled") ;
-                QMessageBox msgBox;
-                msgBox.setWindowTitle(tr("KLog - User cancelled"));
-                QString aux = QString(tr("You have canceled the file import. The file will be removed and no data will be imported.") + "\n" + tr("Do you still want to cancel?"));
-                msgBox.setText(aux);
-                msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
-                msgBox.setDefaultButton(QMessageBox::No);
-                int ret = msgBox.exec();
-                switch (ret) {
-                case QMessageBox::Yes:
-                    // Yes was clicked
+                if (handleCancel())
+                {
                     noMoreQSO = true;
                     break;
-                case QMessageBox::No:
-                    // No Save was clicked
-                    break;
-                default:
-                    // should never be reached
-                    break;
                 }
             }
-            //qDebug() << Q_FUNC_INFO << QString(": Field process finished: ").arg(fieldToAnalyze) ;
         }
-        //qDebug() << Q_FUNC_INFO << QString(": List of fields is empty!") ;
     }
-    //qDebug() << Q_FUNC_INFO << QString(": End of File or no more QSOs") ;
-    file.close ();
-    progress.setValue(qsos);    // Closes the progressDialog
 
-    //qDebug() << Q_FUNC_INFO << " - END";
+    int elapsedSec = std::max(1, startTime.secsTo(QTime::currentTime()));
+   //qDebug() << Q_FUNC_INFO << " - Seconds: " << elapsedSec;
+   //qDebug() << Q_FUNC_INFO << " - QSOs: " << qsos;
+    if (elapsedSec>0)
+       //qDebug() << Q_FUNC_INFO << " - QSOs per second: " << (qsos / elapsedSec);
+
+    file.close();
+    progress.setValue(qsos);
+
     return i;
 }
 
+int FileManager::processQSO(QSO& qso, const QString& _stationCallsign)
+{
+   //qDebug() << Q_FUNC_INFO << " - Start: " << _stationCallsign;
+   //qDebug() << Q_FUNC_INFO << " - QSO: " << qso.getCall();
+    Callsign call1(_stationCallsign);
+    Callsign call2(qso.getStationCallsign());
+    if (call1.isValid() && !call2.isValid())
+    {
+        qso.setStationCallsign(_stationCallsign);
+    }
+    int qsoId = -1;
+    if (qso.getLoTWUpdating())
+    {
+       //qDebug() << Q_FUNC_INFO << " - Running LoTW update code";
+        int bandId = dataProxy->getIdFromBandName(qso.getBand());
+        int modeId = dataProxy->getIdFromModeName(qso.getMode());
+        qsoId = dataProxy->getDuplicatedQSOId(qso.getCall(), qso.getDateTimeOn(), bandId, modeId);
+
+        if (qsoId > 0)
+        {
+            qso.updateFromLoTW(qsoId);
+        }
+    }
+   //qDebug() << Q_FUNC_INFO << " - Ready to add the QSO to the DB";
+    return qso.toDB(qsoId);
+}
+
+bool FileManager::handleCancel()
+{
+    QMessageBox msgBox;
+    msgBox.setWindowTitle(tr("KLog - User cancelled"));
+    msgBox.setText(tr("You have canceled the file import. The file will be removed and no data will be imported.\nDo you still want to cancel?"));
+    msgBox.setStandardButtons(QMessageBox::Yes | QMessageBox::No);
+    msgBox.setDefaultButton(QMessageBox::No);
+
+    return msgBox.exec() == QMessageBox::Yes;
+}
+
+
+QString FileManager::getProgramID (QFile &_f)
+{
+   //qDebug() << Q_FUNC_INFO << " - Start: " << _f.fileName ();
+    QFile &file = _f;
+    if (!file.open(QIODevice::ReadOnly | QIODevice::Text)) /* Flawfinder: ignore */
+    {
+       //qDebug() << Q_FUNC_INFO << "  File not found" ;
+        return QString();
+    }
+    QString line;
+    while ( !file.atEnd()  )
+    {
+        line.clear();
+        line.append(file.readLine().trimmed().toUpper());
+
+        QStringList fields = line.split("<", QT_SKIP);
+        while (!fields.isEmpty())
+        {
+            QString fieldToAnalyze = "<" + (fields.takeFirst()).trimmed();
+            if (fieldToAnalyze.contains ("<PROGRAMID:"))
+            { // <PROGRAMID:4>LOTW
+                file.close();
+                QStringList programIdFields = fieldToAnalyze.split(">");
+               //qDebug() << Q_FUNC_INFO << "  Is a LOTW file: " << programIdFields.at(1);
+                return programIdFields.at(1);
+            }
+        }
+    }
+    file.close();
+    return QString();
+}
 qint64 FileManager::passHeader(QFile & _f)
 {
     //qDebug() << Q_FUNC_INFO << " - Start: " << _f.fileName ();
@@ -1293,7 +995,7 @@ bool FileManager::getStationCallsignFromUser(const QString &_qrzDX, const QDate
     Callsign callsign(_qrzDX);
     if (callsign.isValid())
     {
-        aux = tr("KLog has found one QSO without the Station Callsign defined.\n\nEnter the Station Callsign that was used to do this QSO with %1 on %2:").arg(_qrzDX).arg(_date);
+        aux = tr("KLog has found one QSO without the Station Callsign defined.\n\nEnter the Station Callsign that was used to do this QSO with %1 on %2:").arg(_qrzDX, _date);
         text = QInputDialog::getText(this, tr("KLog - QSO without Station Callsign"),
                                             aux, QLineEdit::Normal, "", &ok);
     }
@@ -1568,79 +1270,6 @@ void FileManager::setVersion(const QString &_version)
     util->setVersion(_version);
 }
 
-QStringList FileManager::readAdifField(const QString &_field)
-{
-    // This function receives a QString with an ADIF field and returns a QStringList with the following format:
-    // ADIF-tag, value
-    // If the QString is not an ADIF tag, it returns a clear QStringList.
-    // We are expecting the ADIF format:
-    // <F:L:T>D
-    // <Field:Length:Type>Data
-
-    //qDebug() << "FileManager::readAdifField: " << _field;
-    QStringList result;
-    result.clear();
-
-    if (_field == "<EOR>")
-    {
-        //qDebug() << "FileManager::readAdifField: EOR found!!";
-        result << "EOR" << "EOR";
-        return result;
-    }
-
-    if (!((_field.startsWith("<")) && (_field.contains(":")) && (_field.contains(">"))))
-    {
-        //qDebug() << "FileManager::readAdifField: NOT (contains : and >): " << _field;
-        return QStringList();
-    }
-
-    // Now we have the data in the result[1]
-    result = _field.split(">", QT_SKIP); // Remove the first '<' and split in field & data
-    QString data = result.at(1);
-    QStringList fieldList;
-    fieldList.clear();
-    fieldList << (result.at(0)).split(':'); // we may have 1 or 2 depending on the format of the ADIF field.
-    int iAux = fieldList.length();
-      //qDebug() << "FileManager::readAdifField: iAux: " << QString::number(iAux);
-    int dataLength;
-    QString field;
-    QString fieldType;
-      //qDebug() << "FileManager::readAdifField: analyzing..." ;
-
-    if (iAux == 2)
-    {
-        field = fieldList.at(0);
-        dataLength = (fieldList.at(1)).toInt();
-        fieldType = QString();
-    }
-    else if (iAux == 3)
-    {
-        field = fieldList.at(0);
-        dataLength = (fieldList.at(1)).toInt();
-        fieldType = fieldList.at(2);
-    }
-    else
-    { // Not valid ADIF
-         //qDebug() << "FileManager::readAdifField: iAux != 1, 2";
-        return QStringList();
-    }
-    field.remove('<');
-    data = data.left(dataLength);
-      //qDebug() << "FileManager::readAdifField: field: " << field;
-      //qDebug() << "FileManager::readAdifField: dataLength: " << QString::number(dataLength);
-      //qDebug() << "FileManager::readAdifField: data: " << data;
-      //qDebug() << "FileManager::readAdifField: fieldType: " << fieldType;
-
-    if (data.length() != dataLength)
-    {
-        //qDebug() << "FileManager::readAdifField: data.length != dataLength: " << QString::number(data.length()) << "/" << QString::number(dataLength);
-        return QStringList();
-    }
-    result.clear();
-    result << field << data;
-    //qDebug() << "FileManager::readAdifField: OK: " << field << "/" << data;
-    return result;
-}
 
 QDateTime FileManager::getDateTimeOfLastBackup()
 {
@@ -1757,7 +1386,7 @@ bool FileManager::askUserToAddThisQSOToLog(const QString &_call, const QDateTime
 {
     Q_UNUSED(_freq);
     //qDebug() << Q_FUNC_INFO << " - " << _call;
-    QString qsoData = QString(tr("<ul><li>Date/Time:</i> %1</li><li>Callsign: %2</li><li>Band: %3</li><li>Mode: %4</li></ul>")).arg(util->getDateTimeSQLiteStringFromDateTime(_datetime)).arg(_call).arg(_band).arg(_mode);
+    QString qsoData = QString(tr("<ul><li>Date/Time:</i> %1</li><li>Callsign: %2</li><li>Band: %3</li><li>Mode: %4</li></ul>")).arg(util->getDateTimeSQLiteStringFromDateTime(_datetime), _call, _band, _mode);
     QMessageBox msgBox;
     msgBox.setTextFormat(Qt::RichText);
     msgBox.setIcon(QMessageBox::Warning);
@@ -1824,7 +1453,7 @@ void FileManager::writeADIFHeader(QTextStream &out, const ExportMode _em, const
     }
     else
     {
-        out << "ADIF v3.1.0 Export from KLog\nhttps://www.klog.xyz/klog\n<PROGRAMVERSION:" << QString::number(klogVersion.length()) << ">" << klogVersion << "\n<PROGRAMID:4>KLOG ";
+        out << "ADIF v3.1.0 Export from KLog\nhttps://github.com/ea4k/klog\n<PROGRAMVERSION:" << QString::number(klogVersion.length()) << ">" << klogVersion << "\n<PROGRAMID:4>KLOG ";
         out << "<APP_KLOG_QSOS:" << QString::number((QString::number(_numberOfQsos)).length()) << ">" << QString::number(_numberOfQsos);
         out << "<APP_KLOG_LOG_DATE_EXPORT:" << QString::number((QDateTime::currentDateTime().toString("yyyyMMdd-hhmm")).length()) << ">" << QDateTime::currentDateTime().toString("yyyyMMdd-hhmm");
     }
