@@ -30,61 +30,56 @@
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
+#include "utilities.h"
 
 HamLibClass::HamLibClass(QObject *parent) : QObject(parent)
 {
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO;
     timer = new QTimer(this);
-    //my_rig = rig_init (RIG_DUMMY);
-    rig_set_debug(RIG_DEBUG_NONE);
-    my_rig = rig_init (RIG_MODEL_DUMMY);
-    retcode = -1;
-    bauds = 9600;
+    my_rig = nullptr;
+    connected = false;
+    connect(timer, &QTimer::timeout, this, &HamLibClass::readFreq);
+
     //qDebug() << Q_FUNC_INFO << " - END";
 }
 
 HamLibClass::~HamLibClass()
 {
     //qDebug() << Q_FUNC_INFO;
-    if (rigLaunched)
-    {
-        rig_close(my_rig);
-        rig_cleanup(my_rig);
-        rigLaunched = false;
-    }
-    //qDebug() << Q_FUNC_INFO << " - END";
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    if (timer)
+        timer->stop();
+   //qDebug() << Q_FUNC_INFO << " - END";
+    cleanup();
+    delete(timer);
 }
 
 void HamLibClass::initClass()
 {
-    //qDebug() << Q_FUNC_INFO ;
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    //qDebug() << Q_FUNC_INFO;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     strings.clear();
     fillRigsList();
-    retcode = -1;
-    rigLaunched = false;
-    pollInterval = 300;
-    errorCount = 0;
-    readOnlyMode = false;
-    justEmitted = false;
-    reading = false;
-    freq_old = 0.0;
-    bauds = 9600;                  // default 9600
-    dataBits = 8;               // default 8
-    stopBits = 1;               // default 1
-    connect(timer, SIGNAL(timeout()), this, SLOT(slotTimer()) );
+    rig_set_debug(RIG_DEBUG_NONE);
+
+    connect(timer, SIGNAL(timeout()), this, SLOT(slotTimer()));
     clean();
     //qDebug() << Q_FUNC_INFO << " - END";
 }
 
 void HamLibClass::clean()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug() << Q_FUNC_INFO ;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << Q_FUNC_INFO;
     myrig_model = 1;        //Dummy equipment
     bauds = 9600;
     dataBits = 8;
     stopBits = 1;
+    pollInterval = 300;
+
+    retcode = -1;
+    readOnlyMode = false;
     shandshake = RIG_HANDSHAKE_NONE;
     sparity = RIG_PARITY_NONE;
     serialPort = QString();
@@ -92,62 +87,107 @@ void HamLibClass::clean()
     srts = RIG_SIGNAL_OFF;
     networkPort = 4532;
     networkAddress = "127.0.0.1";
-    rigLaunched = false;
-    //qDebug() << Q_FUNC_INFO << " - END" ;
+    freq_old = 0.0;
+    connected = false;
+    errorCount = 0;
+    justEmitted = false;
+    reading = false;
+    //qDebug() << Q_FUNC_INFO << " - END";
+}
+
+void HamLibClass::cleanup()
+{
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+
+   //qDebug() << Q_FUNC_INFO << " - Start. Current State: " << static_cast<int>(rig_state);
+
+    // 1. Check the status
+    // Ignore the call if disconnected or in the process to be disconnected
+    if (rig_state == RigState::Disconnected || rig_state == RigState::Disconnecting) {
+       //qDebug() << Q_FUNC_INFO << " - Ignoring: Already disconnected or disconnecting.";
+        return;
+    }
+
+    // 2. Change the status
+    rig_state = RigState::Disconnecting;
+
+    // 3. Cleaning
+    if (timer) {
+        timer->stop();
+    }
+
+   //qDebug() << Q_FUNC_INFO << " - 10";
+
+    if (my_rig) {
+       //qDebug() << Q_FUNC_INFO << " - 40";
+        rig_close(my_rig);
+       //qDebug() << Q_FUNC_INFO << " - 41";
+        rig_cleanup(my_rig);
+       //qDebug() << Q_FUNC_INFO << " - 42";
+        my_rig = nullptr;
+       //qDebug() << Q_FUNC_INFO << " - 43";
+    }
+
+   //qDebug() << Q_FUNC_INFO << " - 50";
+
+    // 4. Update status
+    connected = false; // To be removed when the rest of the class is updated
+    rig_state = RigState::Disconnected;
+
+   //qDebug() << Q_FUNC_INFO << " - END";
 }
 
 void HamLibClass::setPoll(const int _milsecs)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     if (_milsecs>0)
     {
         pollInterval = _milsecs;
     }
 }
 
-double HamLibClass::getFrequency()
+void HamLibClass::readFreq()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug() << Q_FUNC_INFO;
-    if (!isRunning())
-    {
-        //qDebug() << Q_FUNC_INFO << ": isn't running";
-        return 0.0;
-    }
-    if (reading)
-    {
-        //qDebug() << Q_FUNC_INFO << ": Exiting, I was already reading ... ";
-        return 0.0;
-    }
-    reading = true;
-    //qDebug() << Q_FUNC_INFO << ": Reading ... ";
-    //retcode = rig_get_freq(my_rig, RIG_VFO_CURR, &freq);
-    retcode = rig_get_freq(my_rig, RIG_VFO_CURR, &freq);
+   //qDebug() << Q_FUNC_INFO;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    if (!my_rig || !connected)
+        return;
 
-    if (retcode == RIG_OK)
-    {
-        //qDebug() << Q_FUNC_INFO << ": RIG OK for Freq: " << QString::number(freq);
-        errorCount = 0;
-        reading = false;
-        return freq/1000000;
-    }
-    else
-    {
-        reading = false;
-        //qDebug() << Q_FUNC_INFO << " error on readFreq - END";
-        return 0.0;
+   //qDebug() << Q_FUNC_INFO << " - reading freq...";
+    freq_t freq;
+    //mode_t mode;
+    //pbwidth_t width;
+    int ret;
+
+    // 1. Read freq
+    ret = rig_get_freq(my_rig, RIG_VFO_CURR, &freq);
+    if (ret == RIG_OK) {
+        Frequency freqToEmit((double) freq, Hz);
+        if (freqToEmit.isValid()) {
+           //qDebug() << Q_FUNC_INFO << " - freqToEmit valid!";
+            emit freqTXChanged(freqToEmit.toDouble());
+        } else {
+           //qDebug() << Q_FUNC_INFO << " - freqToEmit not valid!";
+        }
+
+    } else {
+        // We should disconnect if there are several continuous errors
+        // logEvent(Q_FUNC_INFO, "Error reading frequency", Debug);
+       //qDebug() << Q_FUNC_INFO << " - Error reading frequency";
     }
+
 }
 
 bool HamLibClass::readRadio(bool _forceRead)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     return readRadioInternal(_forceRead);
 }
 
 bool HamLibClass::readRadioInternal(bool _forceRead)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO;
     if (!isRunning())
     {
@@ -170,7 +210,8 @@ bool HamLibClass::readRadioInternal(bool _forceRead)
         //qDebug() << Q_FUNC_INFO << ": Freq: " << QString::number(freq);
         if ((freq_old > freq) || (freq_old < freq) || (_forceRead == true))
         {
-            emit freqChanged(freq/1000000);
+            Frequency freqToEmit((double) freq, Hz);
+            emit freqTXChanged(freqToEmit.toDouble());
             freq_old = freq;
             //qDebug() << Q_FUNC_INFO << ": EMITING; " << QString::number(freq);
         }
@@ -199,7 +240,7 @@ bool HamLibClass::readRadioInternal(bool _forceRead)
                 return true;
             }
             mode_old = rmode;
-            //qDebug() << Q_FUNC_INFO << " - Emmiting mode: " << hamlibMode2Mode(rmode) ;
+            //qDebug() << Q_FUNC_INFO << " - Emmiting mode: " << hamlibMode2Mode(rmode);
             emit modeChanged(hamlibMode2Mode(rmode));
             justEmitted = true;
         }
@@ -217,21 +258,21 @@ bool HamLibClass::readRadioInternal(bool _forceRead)
 
 void HamLibClass::slotTimer()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-     //qDebug() << Q_FUNC_INFO;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << Q_FUNC_INFO;
     if (!isRunning())
     {
-         //qDebug() << Q_FUNC_INFO << ": Isn't running...";
+        //qDebug() << Q_FUNC_INFO << ": Isn't running...";
         return;
     }
     readRadioInternal(false); // We don't force the radio reading.
-     //qDebug() << Q_FUNC_INFO << " - END";
+    //qDebug() << Q_FUNC_INFO << " - END";
 }
 
 void HamLibClass::setMode(const QString &_m)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug() << "HamLibClass::setMode: " << _m ;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << "HamLibClass::setMode: " << _m;
     if ((!isRunning()) || (readOnlyMode))
     {
         //qDebug() << Q_FUNC_INFO << ": Not running or RO";
@@ -269,7 +310,7 @@ void HamLibClass::setMode(const QString &_m)
 
 bool HamLibClass::isModeADIFMode(const QString &_m)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     QString _mode;
     _mode = _m.toUpper();
     if ((_mode == "AM") ||  (_mode == "CW") || (_mode == "FM") || (_mode == "LSB") ||
@@ -285,7 +326,7 @@ bool HamLibClass::isModeADIFMode(const QString &_m)
 
 QString HamLibClass::hamlibMode2Mode(rmode_t _rmode)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     switch (_rmode)
     {
         case RIG_MODE_NONE:
@@ -340,7 +381,7 @@ QString HamLibClass::hamlibMode2Mode(rmode_t _rmode)
 
 rmode_t HamLibClass::mode2HamlibMode (const QString &_mode)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _mode;
     if (_mode == "USB")
     {
@@ -381,212 +422,168 @@ rmode_t HamLibClass::mode2HamlibMode (const QString &_mode)
 
 bool HamLibClass::stop()
 {
-    //qDebug() << Q_FUNC_INFO;
-    timer->stop();
-    if (!isRunning())
-    {
-        //qDebug() << Q_FUNC_INFO << " - ALready stopped - END";
-        return true;
-    }
-    int errorCode = rig_close(my_rig);
-    //qDebug() << Q_FUNC_INFO << " - 10";
-
-    if (errorCode == RIG_OK)
-    {
-        errorCount = 0;
-        //qDebug() << Q_FUNC_INFO << " rig_close OK";
-        errorCode = rig_cleanup(my_rig);
-        if (errorCode == RIG_OK)
-        {
-            errorCount = 0;
-            //qDebug() << Q_FUNC_INFO << " rig_cleanUp OK";
-            rigLaunched = false;
-            //qDebug() << Q_FUNC_INFO << " - END true";
-            return true;
-        }
-        else
-        {
-            //return errorManage(Q_FUNC_INFO,  retcode);
-            errorCount++;
-            //qDebug() << Q_FUNC_INFO << " rig_cleanup NOK: " << QString::number(errorCode);
-        }
-    }
-    else
-    {
-        errorCount++;
-        //qDebug() << Q_FUNC_INFO << " rig_close NOK: " << QString::number(errorCode);
-    }
-    //qDebug() << Q_FUNC_INFO << " - END false";
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+   //qDebug() << Q_FUNC_INFO << " - Start";
+    cleanup();
+   //qDebug() << Q_FUNC_INFO << " - END";
     return false;
 }
 
 bool HamLibClass::init(bool _active)
 {
-   //qDebug()<< Q_FUNC_INFO << ": " << getNameFromModelId(myrig_model);
-    if (!loadSettings())
-       return false;
-    if (!_active)
-    {
-       //qDebug()<< Q_FUNC_INFO << ": not active, exiting";
-        rigLaunched = false;
-        stop();
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+   //qDebug() << Q_FUNC_INFO << " - Start Initialization";
+
+    // 1. BARRERA MEJORADA (Reconexión dinámica)
+    // If connected we clean it first
+    if (rig_state == RigState::Connected || rig_state == RigState::Connecting) {
+       //qDebug() << Q_FUNC_INFO << " - Reconnecting: Closing previous connection first...";
+        cleanup();
+        // Al terminar cleanup(), rig_state pasará a ser Disconnected y my_rig a nullptr.
+        // Así podemos continuar de forma 100% segura hacia abajo.
+    }
+
+    if (myrig_model <= 1) { // 0 = Dummy, 1 = Invalid
+       //qDebug() << Q_FUNC_INFO << " - ERROR: Hamlib disabled or invalid model.";
         return false;
     }
 
+    rig_state = RigState::Connecting;
+   //qDebug() << Q_FUNC_INFO << " - 10: Initializing rig model: " << myrig_model;
 
-
-    if ((getNameFromModelId(myrig_model)).length ()<1)
-    {
-       //qDebug()<< Q_FUNC_INFO << ": no rig model!";
-        rigLaunched = false;
-        return false;
-    }
-
-    if (rigLaunched)
-    {
-       //qDebug()<< Q_FUNC_INFO << ": Already init" ;
-        return true;
-    }
-
+    // 2. Instanciar la radio
     my_rig = rig_init(myrig_model);
-   //qDebug()<< Q_FUNC_INFO << ": set after init" ;
-    //if (my_rig == nullptr)
-    if (!my_rig)
-    {
-      //qDebug()<< Q_FUNC_INFO << ": Init failed, hamlib returned fail!";
-       return false;
+    if (!my_rig) {
+       //qDebug() << Q_FUNC_INFO << " - ERROR: Failed to initialize Rig (rig_init returned NULL).";
+        rig_state = RigState::Disconnected;
+        return false;
     }
-    else
-    {
-       //qDebug()<< Q_FUNC_INFO << ": rig_init went OK!";
-    }
-    // Code of DG1VS (Thank you!)
-    if (myrig_model == RIG_MODEL_NETRIGCTL)
-    {
-       //qDebug()<< Q_FUNC_INFO << ": RIG_PORT_NETWORK";
-        // network based communication
+
+   //qDebug() << Q_FUNC_INFO << " - 20: Configuring port";
+
+    // 3. Configurar el puerto dependiendo de si es Red (Rigctl) o Serie local
+    if (myrig_model == RIG_MODEL_NETRIGCTL) {
+        // --- CONFIGURACIÓN DE RED ---
+        // Hamlib espera el formato "IP:PORT" en el pathname
+        QString netPath = QString("%1:%2").arg(networkAddress).arg(networkPort);
+        strncpy(my_rig->state.rigport.pathname, netPath.toLatin1().constData(), FILPATHLEN - 1);
         my_rig->state.rigport.type.rig = RIG_PORT_NETWORK;
-        QString netAddPort = QString("%1:%2").arg (networkAddress).arg(networkPort);
-        qstrncpy (my_rig->state.rigport.pathname, netAddPort.toLocal8Bit().constData(), FILPATHLEN);
-        // the other stuff is hardcoded in hamlib!
     }
-    else if (myrig_model == RIG_MODEL_FLRIG)
-    {
-       //qDebug()<< Q_FUNC_INFO << ": RIG_PORT_RPC";
-        my_rig->state.rigport.type.rig = RIG_PORT_RPC;
-        //my_rig->state.rigport.type.rig = RIG_PORT_NETWORK;
-        QString netAddPort = QString("%1:%2").arg (networkAddress).arg(networkPort);
-       //qDebug()<< Q_FUNC_INFO << ": " << netAddPort;
-        qstrncpy (my_rig->state.rigport.pathname, netAddPort.toLocal8Bit().constData(), FILPATHLEN);
-    }
-    else if ((myrig_model == RIG_MODEL_DUMMY) || (myrig_model == RIG_MODEL_DUMMY_NOVFO))
-    {
-       //qDebug()<< Q_FUNC_INFO << ": RIG_PORT_DUMMY";
-    }
-    else
-    {
-       //qDebug()<< Q_FUNC_INFO << ": !RIG_PORT_NETWORK";
-       //qDebug()<< Q_FUNC_INFO << ": serialport2: " << serialPort.toLocal8Bit();
-        my_rig->state.rigport.type.rig = RIG_PORT_SERIAL;
-        //qstrncpy (my_rig->state.rigport.pathname, serialPort.toLocal8Bit().constData(), FILPATHLEN);
-        QVariant aux = QVariant(serialPort);
-        QByteArray portStr = aux.toByteArray();
-        const char* port = portStr.constData();
-        strncpy(my_rig->state.rigport.pathname, port, FILPATHLEN - 1);
-       //qDebug()<< Q_FUNC_INFO << ": rigport: " << my_rig->state.rigport.pathname ;
-        my_rig->state.rigport.parm.serial.rate = bauds;
-       //qDebug()<< Q_FUNC_INFO << ": serial rate: " << QString::number(my_rig->state.rigport.parm.serial.rate) ;
-        my_rig->state.rigport.parm.serial.data_bits = dataBits;
-       //qDebug()<< Q_FUNC_INFO << ": data bits: " << QString::number(my_rig->state.rigport.parm.serial.data_bits) ;
-        my_rig->state.rigport.parm.serial.stop_bits = stopBits;
-       //qDebug()<< Q_FUNC_INFO << ": stop bits: " << QString::number(my_rig->state.rigport.parm.serial.stop_bits);
-        my_rig->state.rigport.parm.serial.parity = sparity;
-       //qDebug()<< Q_FUNC_INFO << ": handshake before" ;
-        my_rig->state.rigport.parm.serial.handshake = shandshake;
-      //qDebug()<< Q_FUNC_INFO << ": after handshake " ;
-        // Config done
-    }
-   //qDebug()<< Q_FUNC_INFO << ": Rig model config " ;
-    // Config done
-    retcode = rig_open(my_rig);
-   //qDebug()<< Q_FUNC_INFO << ": retcode: " << QString::number(retcode) ;
+    else {
+        // --- CONFIGURACIÓN SERIE (USB/COM) ---
+        if (!serialPort.isEmpty()) {
+            strncpy(my_rig->state.rigport.pathname, serialPort.toLatin1().constData(), FILPATHLEN - 1);
+            my_rig->state.rigport.type.rig = RIG_PORT_SERIAL;
+        } else {
+           //qDebug() << Q_FUNC_INFO << " - ERROR: Serial port is empty!";
+            // Fallo intencionado si no hay puerto, no vale la pena seguir
+            rig_cleanup(my_rig);
+            my_rig = nullptr;
+            rig_state = RigState::Disconnected;
+            return false;
+        }
 
-    if (retcode != RIG_OK)
-    {
-       //qDebug()<< Q_FUNC_INFO << ": Can't open: " << rigerror(retcode);
-        rig_cleanup(my_rig);
-        return errorManage(Q_FUNC_INFO,  retcode);
-    }
-   //qDebug()<< Q_FUNC_INFO << ": Rig open!" ;
-    errorCount = 0;
-    rigLaunched = true;
-    freq_old = 0.0;
-    timer->start(pollInterval);
+        // Configurar la velocidad de forma nativa a través de Hamlib
+        if (bauds > 0) {
+            rig_set_conf(my_rig, rig_token_lookup(my_rig, "serial_speed"), QString::number(bauds).toLatin1().constData());
+        }
 
-   //qDebug()<< Q_FUNC_INFO << ": END TRUE";
-    return true;
+        // (Opcional) Si quieres pasar el resto de parámetros (Paridad, DataBits, etc)
+        // rig_set_conf(my_rig, rig_token_lookup(my_rig, "data_bits"), QString::number(dataBits).toLatin1().constData());
+        // rig_set_conf(my_rig, rig_token_lookup(my_rig, "stop_bits"), QString::number(stopBits).toLatin1().constData());
+    }
+
+   //qDebug() << Q_FUNC_INFO << " - 30: Opening connection";
+
+    // 4. Abrir la conexión real
+    int retcode = rig_open(my_rig);
+
+    if (retcode == RIG_OK) {
+       //qDebug() << Q_FUNC_INFO << " - SUCCESS: Rig opened";
+        rig_state = RigState::Connected;
+        connected = true;
+
+        // ¡IMPORTANTE! Iniciar el polling si la conexión tuvo éxito
+        if (_active && timer) {
+            timer->start(pollInterval);
+        }
+        return true;
+
+    } else {
+       //qDebug() << Q_FUNC_INFO << " - ERROR: rig_open failed with code: " << retcode;
+        rig_state = RigState::Error;
+        connected = false;
+
+        // Limpieza profunda tras fallo
+        if (my_rig) {
+            rig_cleanup(my_rig);
+            my_rig = nullptr;
+        }
+        return false;
+    }
 }
 
 bool HamLibClass::isRunning()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    return rigLaunched;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    return connected;
 }
 
 void HamLibClass::fillRigsList()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug()<< "HamLibClass::getRigList: StringsList before filling it: ";
-  // Rutine to fill the rig combo boxes
-  // Do not display debug codes when load the rig's
-  //rig_set_debug (RIG_DEBUG_NONE);
-     //qDebug() << "HamLibClass::getRigList-01";
-  // and continue...
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << "HamLibClass::getRigList: StringsList before filling it: ";
+    // Rutine to fill the rig combo boxes
+    // Do not display debug codes when load the rig's
+    //rig_set_debug (RIG_DEBUG_NONE);
+    //qDebug() << "HamLibClass::getRigList-01";
+    // and continue...
 
-  strings.clear();
-  //qDebug()<< "HamLibClass::getRigList: StringsList after cleaning it: ";
-   //qDebug() << "HamLibClass::getRigList-02";
-  rig_load_all_backends();
-   //qDebug() << "HamLibClass::getRigList-10";
-  rig_list_foreach (addRigToList, this);
-     //qDebug() << "HamLibClass::getRigList-11";
-  strings.sort();
+    strings.clear();
+    //qDebug() << "HamLibClass::getRigList: StringsList after cleaning it: ";
+    //qDebug() << "HamLibClass::getRigList-02";
+    rig_load_all_backends();
+    //qDebug() << "HamLibClass::getRigList-10";
+    rig_list_foreach(addRigToList, this);
+    //qDebug() << "HamLibClass::getRigList-11";
+    strings.sort();
 }
 
-QStringList HamLibClass::getRigList ()
+QStringList HamLibClass::getRigList()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug()<< "HamLibClass::getRigList: StringsList before filling it: ";
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << "HamLibClass::getRigList: StringsList before filling it: ";
 
     //fillRigsList ();
     //strings.sort();
-    //qDebug()<< "HamLibClass::getRigList-12 - Strings length: " << QString::number(strings.length());
+    //qDebug() << "HamLibClass::getRigList-12 - Strings length: "
+    //<< QString::number(strings.length());
     return strings;
  }
 
 int HamLibClass::addRigToList (const struct rig_caps *caps, void *data)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug()<< "HamLibClass::addRigToList" << caps->model_name;
+    //logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << "HamLibClass::addRigToList" << caps->model_name;
     QString name;
-    //qDebug()<< "HamLibClass::addRigToList-10" ;
+    //qDebug() << "HamLibClass::addRigToList-10";
     //HamLibClass *r = (HamLibClass *) data;
     HamLibClass *r = static_cast<HamLibClass *> (data);
-    //qDebug()<< "HamLibClass::addRigToList-11" ;
+    //qDebug() << "HamLibClass::addRigToList-11";
     name = caps->model_name;
-    //qDebug()<< "HamLibClass::addRigToList-12" ;
+    //qDebug() << "HamLibClass::addRigToList-12";
     r->rigName2RigId[name] = caps->rig_model; // We fill the equivalences between name & Id
-    //qDebug()<< "HamLibClass::addRigToList-13: " <<  caps->rig_model;
+    //qDebug() << "HamLibClass::addRigToList-13: " << caps->rig_model;
     r->rigId2RigName[caps->rig_model] = name;
-    //qDebug()<< "HamLibClass::addRigToList-14: " << name;
+    //qDebug() << "HamLibClass::addRigToList-14: " << name;
     r->strings << name;
-    //qDebug()<< "HamLibClass::addRigToList-END" ;
+    //qDebug() << "HamLibClass::addRigToList-END";
     return -1;                    // not 0 --> we want all rigs
 }
 
 int HamLibClass::getModelIdFromName (const QString &_name)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
    //HamLibClass *r (HamLibClass *) data;
    int i = -1;
    i = rigName2RigId[_name];
@@ -596,42 +593,43 @@ int HamLibClass::getModelIdFromName (const QString &_name)
 
 QString HamLibClass::getNameFromModelId(const int _id)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug() << "HamLibClass::getNameFromModelId: " << QString::number(_id) << "/" << rigId2RigName.value(_id) ;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << "HamLibClass::getNameFromModelId: " << QString::number(_id) << "/"
+    // << rigId2RigName.value(_id);
     return rigId2RigName.value(_id);
 }
 
 void HamLibClass::setModelId(const int _id)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << "HamLibClass::setModelId: " << QString::number(_id);
-    rigLaunched = false;
+    connected = false;
     myrig_model = _id;
 }
 
 void HamLibClass::setPort(const QString &_port)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << "HamLibClass::setPort: " << _port;
     serialPort = _port;
-    rigLaunched = false;
+    connected = false;
     //strncpy (my_rig->state.rigport.pathname, serialPort.toLocal8Bit().constData(), FILPATHLEN);
-    //qstrncpy(myport.pathname, serialPort.toLocal8Bit().constData(), serialPort.length()+1);
+    // qstrncpy(myport.pathname, serialPort.toLocal8Bit().constData(), serialPort.length()+1);
 }
 
 void HamLibClass::setSpeed(const int _speed)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //TODO: Check that it is a valid speed
     bauds = _speed;
-    rigLaunched = false;
+    connected = false;
     //qDebug() << Q_FUNC_INFO << ": " << QString::number(bauds);
 }
 
 void HamLibClass::setDataBits(const int _data)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug() << Q_FUNC_INFO << ": rec: " << QString::number(_data);
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    ////qDebug() << Q_FUNC_INFO << ": rec: " << QString::number(_data);
     if ((_data >= 5) && (_data <= 8))
     {
         dataBits = _data;
@@ -640,14 +638,14 @@ void HamLibClass::setDataBits(const int _data)
     {
         dataBits = 8;
     }
-    rigLaunched = false;
-   //qDebug() << Q_FUNC_INFO << ": final: " << QString::number(dataBits);
+    connected = false;
+    //qDebug() << Q_FUNC_INFO << ": final: " << QString::number(dataBits);
 }
 
 void HamLibClass::setStop(const QString &_stop)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-    //qDebug() << Q_FUNC_INFO << ": " << _stop;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    ////qDebug() << Q_FUNC_INFO << ": " << _stop;
 
     if (_stop == "OneStop")
     {
@@ -665,12 +663,12 @@ void HamLibClass::setStop(const QString &_stop)
     {
        stopBits = -1;
     }
-    rigLaunched = false;
+    connected = false;
 }
 
 void HamLibClass::setFlow(const QString &_flow)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _flow;
     flowControl = _flow.toUpper();
 
@@ -686,12 +684,12 @@ void HamLibClass::setFlow(const QString &_flow)
     {
         shandshake = RIG_HANDSHAKE_NONE;
     }
-    rigLaunched = false;
+    connected = false;
 }
 
 void HamLibClass::setParity(const QString &_parity)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _parity;
     parity = _parity.toUpper();
     if (parity == "EVEN")
@@ -714,19 +712,18 @@ void HamLibClass::setParity(const QString &_parity)
     {
         sparity = RIG_PARITY_NONE;
     }
-    rigLaunched = false;
+    connected = false;
 }
 
-void HamLibClass::setFreq(const double _fr)
+void HamLibClass::setFreq(const Frequency &_fr)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-     //qDebug() << "HamLibClass::setFreq: " << QString::number(_fr);
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << "HamLibClass::setFreq: " << QString::number(_fr);
     if ((!isRunning()) || (readOnlyMode))
     {
         return;
     }
-
-    freq = _fr * 1000000;
+    freq = _fr.toDouble (Hz);
     int retcode = rig_set_freq(my_rig, RIG_VFO_CURR, freq);
     if (retcode != RIG_OK)
     {
@@ -735,12 +732,12 @@ void HamLibClass::setFreq(const double _fr)
     else
     {
         errorCount = 0;
-         //qDebug() << "HamLibClass::setFreq OK: " << QString::number(freq);
+        //qDebug() << "HamLibClass::setFreq OK: " << QString::number(freq);
         retcode = rig_get_freq(my_rig, RIG_VFO_CURR, &freq);
         if (retcode == RIG_OK)
         {
             errorCount = 0;
-             //qDebug() << "HamLibClass::setFreq read: " << QString::number(freq);
+            //qDebug() << "HamLibClass::setFreq read: " << QString::number(freq);
         }
         else
         {
@@ -752,7 +749,7 @@ void HamLibClass::setFreq(const double _fr)
 
 void HamLibClass::setRTS(const QString &_state)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _state;
     if (shandshake == RIG_HANDSHAKE_HARDWARE)
     { // Note: An attempt to control the RTS signal in the HardwareControl mode will fail with error code
@@ -768,12 +765,12 @@ void HamLibClass::setRTS(const QString &_state)
     {
         srts = RIG_SIGNAL_OFF;
     }
-    rigLaunched = false;
+    connected = false;
 }
 
 void HamLibClass::setDTR(const QString &_state)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _state;
     if (_state.toUpper() == "TRUE")
     {
@@ -783,48 +780,48 @@ void HamLibClass::setDTR(const QString &_state)
     {
         sdtr = RIG_SIGNAL_OFF;
     }
-    rigLaunched = false;
+    connected = false;
 }
 
 void HamLibClass::checkErrorCountAndStop()
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
-     //qDebug() << Q_FUNC_INFO;
+    logEvent(Q_FUNC_INFO, "Start", Debug);
+    //qDebug() << Q_FUNC_INFO;
     if (errorCount > 10)
     {
-         //qDebug() << Q_FUNC_INFO << ": Error>10 - calling stop";
+        //qDebug() << Q_FUNC_INFO << ": Error>10 - calling stop";
         stop();
     }
 }
 
 void HamLibClass::setReadOnly(const bool _r)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     readOnlyMode = _r;
 }
 
 void HamLibClass::setNetworkAddress(const QString &_address)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _address;
     networkAddress = _address;
-    rigLaunched = false;
+    connected = false;
 }
 
 void HamLibClass::setNetworkPort(const int _port)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << QString::number(_port);
     if ((_port>0) && (_port<65535))
     {
         networkPort = _port;
     }
-    rigLaunched = false;
+    connected = false;
 }
 
 bool HamLibClass::errorManage(const QString &_func, const int _errorcode)
 {
-    //showDebugLog(Q_FUNC_INFO, "Start");
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     //qDebug() << Q_FUNC_INFO << ": " << _func << ": error: " << QString::number(_errorcode);
     Q_UNUSED(_func);
     if (RIG_OK == _errorcode)
@@ -836,61 +833,63 @@ bool HamLibClass::errorManage(const QString &_func, const int _errorcode)
     {
     case (RIG_EINVAL):
         //qDebug() << Q_FUNC_INFO << ": Error: 1 invalid parameter";
-    break;
+        break;
     case (RIG_ECONF):
         //qDebug() << Q_FUNC_INFO << ": Error: 2 invalid configuration (serial,..) ";
-    break;
+        break;
     case (RIG_ENOMEM):
         //qDebug() << Q_FUNC_INFO << ": Error: 3 memory shortage";
-    break;
+        break;
     case (RIG_ENIMPL):
         //qDebug() << Q_FUNC_INFO << ": Error: 4 function not implemented, but will be ";
-    break;
+        break;
     case (RIG_ETIMEOUT):
         //qDebug() << Q_FUNC_INFO << ": Error: 5 communication timed ou";
-    break;
+        break;
     case (RIG_EIO):
         //qDebug() << Q_FUNC_INFO << ": Error: 6 IO error, including open failed";
-    break;
+        break;
     case (RIG_EINTERNAL):
         //qDebug() << Q_FUNC_INFO << ": Error: 7 Internal Hamlib error, huh!";
-    break;
+        break;
     case (RIG_EPROTO):
         //qDebug() << Q_FUNC_INFO << ": Error: 9 Command rejected by the rig";
-    break;
+        break;
     case (RIG_ETRUNC):
         //qDebug() << Q_FUNC_INFO << ": Error: 10 Command performed, but arg truncated";
-    break;
+        break;
     case (RIG_ENAVAIL):
         //qDebug() << Q_FUNC_INFO << ": Error: 11 Function not available";
-    break;
+        break;
     case (RIG_ENTARGET):
         //qDebug() << Q_FUNC_INFO << ": Error: 12 VFO not targetable";
-    break;
+        break;
     case (RIG_BUSERROR):
         //qDebug() << Q_FUNC_INFO << ": Error: 13 Error talking on the bus";
-    break;
+        break;
     case (RIG_BUSBUSY):
         //qDebug() << Q_FUNC_INFO << ": Error: 14 Collision on the bus";
-    break;
+        break;
     case (RIG_EARG):
-        //qDebug() << Q_FUNC_INFO << ": Error: 15 NULL RIG handle or any invalid pointer parameter in get arg";
-    break;
+        //qDebug() << Q_FUNC_INFO
+        // << ": Error: 15 NULL RIG handle or any invalid pointer parameter in get arg";
+        break;
     case (RIG_EVFO):
         //qDebug() << Q_FUNC_INFO << ": Error: 16 Invalid VFO";
-    break;
+        break;
     case (RIG_EDOM):
         //qDebug() << Q_FUNC_INFO << ": Error: 17 Argument out of domain of func";
-    break;
+        break;
     default:
-      //qDebug() << Q_FUNC_INFO << ": Error: ?? Unknown error";
-    break;
+        //qDebug() << Q_FUNC_INFO << ": Error: ?? Unknown error";
+        break;
     }
     if (_errorcode == RIG_EINVAL || _errorcode == RIG_ENIMPL || _errorcode == RIG_ERJCTED \
             || _errorcode == RIG_ETRUNC || _errorcode == RIG_ENAVAIL || _errorcode == RIG_ENTARGET \
             || _errorcode == RIG_EVFO || _errorcode == RIG_EDOM)
     {
-        //qDebug() << Q_FUNC_INFO << ": Soft error: Invalid parameters - No reason to re-initialize the hardware";
+        //qDebug() << Q_FUNC_INFO
+        // << ": Soft error: Invalid parameters - No reason to re-initialize the hardware";
     }
 
     if (errorCount<10)
@@ -901,29 +900,20 @@ bool HamLibClass::errorManage(const QString &_func, const int _errorcode)
     }
     else
     {
-         //qDebug() << Q_FUNC_INFO << ": RIG NOK for Mode";
-         //qDebug() << Q_FUNC_INFO << ": Calling stop";
+        //qDebug() << Q_FUNC_INFO << ": RIG NOK for Mode";
+        //qDebug() << Q_FUNC_INFO << ": Calling stop";
         stop();
         return false;
     }
 }
 
-/*void HamLibClass::showDebugLog(const QString &_func, const QString &_log)
-{
-#ifdef QT_DEBUG
-  //qDebug() << _func << ": " << _log;
-#else
-#endif
-}
-*/
-
 bool HamLibClass::loadSettings()
 {
-    //qDebug() << Q_FUNC_INFO << " - Start";
+    logEvent(Q_FUNC_INFO, "Start", Debug);
     Utilities util(Q_FUNC_INFO);
     QSettings settings(util.getCfgFile (), QSettings::IniFormat);
     settings.beginGroup ("HamLib");
-    setModelId(settings.value ("HamLibRigType").toInt());
+    setModelId(settings.value("HamLibRigType", 0).toInt());
     setPort(settings.value ("HamlibSerialPort").toString());
     setSpeed(settings.value ("HamlibSerialBauds", 9600).toInt ());
     setDataBits(settings.value ("HamLibSerialDataBits", 8).toInt ());
@@ -935,5 +925,12 @@ bool HamLibClass::loadSettings()
     setNetworkAddress (settings.value ("HamlibNetAddress").toString());
     setNetworkPort (settings.value ("HamlibNetPort", 4532).toInt ());
     settings.endGroup ();
+    if (pollInterval < 250)
+        pollInterval = 250; // Limit saturation
     return true;
 }
+
+void HamLibClass::logEvent(const QString &_func, const QString &_msg,  DebugLogLevel _level)
+{
+    emit debugLog (_func, _msg, _level);
+}
