@@ -44,28 +44,44 @@ public:
     void addQSO(const QString &_loc);
     void addMarker(const Coordinate _coord);
     void addLocator(const QString &_loc, const QColor &_color);
-    void clearMap();
-    //void setLocale (const QString _locale);
+
+    // Clears only data overlays (worked/confirmed rectangles and circles), NOT the base grid layer
+    void clearDataLayers();
 
 signals:
-      void doAddMarker(double latitude, double longitude);
-
-private slots:
+    void doAddMarker(double latitude, double longitude);
 
 private:
     void createUI();
+
+    // Build the global 2-letter Maidenhead field grid (drawn once, persistent)
+    void paintFieldGrid();
+
     QQuickView qmlView;
 
-    QStandardItemModel modelCircle, modelRectangle;
+    // Models
+    QStandardItemModel modelCircle;      // optional circle markers
+    QStandardItemModel modelRectangle;   // data overlays (worked/confirmed)
+    QStandardItemModel modelGrid;        // persistent field grid (2-letter)
+
+    // Role name maps
     QHash<int, QByteArray> circleRoles;
     QHash<int, QByteArray> rectangleRoles;
+    QHash<int, QByteArray> gridRoles;
 
+    // Roles (keep distinct spaces)
     int CoordinateRole = Qt::UserRole + 1000;
+
+    // For overlays (rectangles)
     int NorthRole = Qt::UserRole + 1000;
     int SouthRole = Qt::UserRole + 1001;
     int ColorRole = Qt::UserRole + 1002;
+
+    // For grid rectangles
+    int GridNorthRole = Qt::UserRole + 1200;
+    int GridSouthRole = Qt::UserRole + 1201;
+
     Locator locator;
-    //double lat, lon;
 };
 
 #endif // MAPWIDGET_H
